/*!
// @file		EmaiSMTP.h
//
// @brief		SMTP functionality
//
// @version		Mailit for C/C++ 1.0
//
// @author		(C) 2008-2010 Dacons LLP.
//
// @bug			If you have problems or suggestions, please contact
//				the Online Support at: http://www.mailitcomponents.com/support
*/

#ifndef __EMAISMTP_H__
#define __EMAISMTP_H__

#include <EmaiTypes.h>
#include <EmaiErrors.h>
#include <EmaiMessage.h>
#include <EmaiProxy.h>

#ifdef __cplusplus
	extern "C" {
#endif

#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef struct EmaiOpaqueSmtpSession*			EmaiSmtpSession;
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

/***************************************************
//
// Functions for communicating with SMTP server
//
***************************************************/

/*!
// @brief	Constants for selector member of the EmaiSmtpCallbackData structure/class.
//
// @see		EmaiSmtpCallbackData
// @see		Emai::SmtpStatusCallback
*/
enum EmaiEnumSmtpSelector
{
	EmaiSmtpSelectorNormal		= 0,/*!< Normal*/
	EmaiSmtpSelectorSending,		/*!< Sending*/
	EmaiSmtpSelectorServerError,	/*!< ServerError*/
};

/*!
// @brief	Constants for operation codes for smtp sessions
//
// @see		EmaiSmtpCallbackData
*/
enum EmaiEnumSmtpOperationCode
{
	EmaiSmtpConnectingToServer		= 0,
	EmaiSmtpConnectedToServer,
	EmaiSmtpAuthenticating,
	EmaiSmtpAuthenticated,
	EmaiSmtpIntroducingSender,
	EmaiSmtpSenderIntroduced,
	EmaiSmtpSendingRecipients,
	EmaiSmtpRecipientsSent,
	EmaiSmtpSendingMessageData,
	EmaiSmtpMessageDataSent,
	EmaiSmtpSendingNOOP,
	EmaiSmtpNOOPSent,
	EmaiSmtpClosingConnection,
	EmaiSmtpConnectionClosed
};

/*!
// @brief	EmaiSmtpCallbackData structure
//
// @see		EmaiEnumSmtpSelector
*/
typedef struct
{
	/*! Version of the structure,*/
	/*! currently EmaiStructureVersionFirst*/
	EmaiUint32					version;

	/*! Selector*/
	EmaiUint32					selector;
	union
	{
		/*! selector == EmaiSmtpSelectorNormal*/
		struct
		{
			EmaiUint32			operationCode;/*!< @see EmaiEnumSmtpOperationCode*/
		} normal;

		/*! selector == EmaiSmtpSelectorSending*/
		struct
		{
			EmaiUint32			sentSize;
			EmaiUint32			totalSize;
		} sending;

		/*! selector == EmaiSmtpSelectorServerError*/
		struct
		{
			EmaiConstCharPtr	serverReply;
			EmaiUint32			replyCode;
		} serverError;
	} data;
} EmaiSmtpCallbackData;


/*
// Status callback declaration for SMTP session.
*/
#ifndef DOXYGEN_SHOULD_SKIP_THIS
typedef EMAICALLBACKAPI(EmaiError, EmaiSmtpStatusCallback)(
		EmaiSmtpSession inSession,
		const EmaiSmtpCallbackData* inData,
		void* inUserData);
#endif /* DOXYGEN_SHOULD_SKIP_THIS */

/*!
// @brief	Constants, used for session establishing function.
//
// @see		EmaiSmtpSessionCreate
// @see		Emai::SmtpSession::SmtpSession
*/
enum EmaiEnumSmtpSSL
{
	EmaiSmtpUseSSLOption	= 0x01, /*!< If this flag is used, this session */
								/* will connect only to secured SMTP servers.*/
};

/*!
// @brief	Creates SMTP session object.
//
// @param	outSession - on exit contains session object, to be used with other function calls
// @param	inStatusCallback - pointer to the callback function (used for status report)
// @param	inUserData - TODO
// @param	inTimeout - timeout in seconds
// @param	inOptions - use EmaiUseSSLOption option to create secured
//					SSL session, or EmaiNullOptions otherwise
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpSessionCreate(
		EmaiSmtpSession* outSession,
		EmaiSmtpStatusCallback inStatusCallback,
		void* inUserData,
		EmaiUint32 inTimeout,
		EmaiOptions inOptions);


/*!
// @brief	Constants for default SMTP port.
//
// @see		EmaiSmtpConnect
// @see		Emai::SmtpSession::Connect
*/
enum EmaiEnumSmtpPorts
{
	EmaiDefaultSmtpPort = 25,		/*!< Default SMTP port*/
	EmaiDefaultSmtpSSLPort = 465	/*!< Default SSL SMTP port*/
};

/*!
// @brief	Connects SMTP session object to the specified mail server.
//
// @param	inSession - session object to be connected
// @param	inMailServer - pointer to null-teminated string, which
//					contains the name or IP address of the SMTP server
// @param	inPort - port number of the SMTP server
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiEnumSmtpPorts
// @see		Emai::SmtpSession::Connect
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpConnect(
		EmaiSmtpSession inSession,
		EmaiConstUniCharPtr inMailServer,
		short inPort,
		EmaiOptions inOptions);

/*!
// @brief	Connects SMTP session object to the specified mail server.
//
// @param	inSession - session object to be connected
// @param	inMailServer - pointer to null-teminated string, which
//					contains the name or IP address of the SMTP server
// @param	inPort - port number of the SMTP server
// @param	inProxyInfo - proxy server information struct
// @param	inUserData - user data specified parameter, use NULL
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		EmaiEnumSmtpPorts
// @see		Emai::SmtpSession::ConnectEx
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpConnectEx(
		EmaiSmtpSession inSession,
		EmaiConstUniCharPtr inMailServer,
		short inPort,
		const EmaiProxyInfo* inProxyInfo,
		void* inUserData,
		EmaiOptions inOptions);

/*!
// @brief	Disconnects SMTP session object from the mail server.
//
// @param	inSession - session object to be disconnected
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::SmtpSession::Disconnect
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpDisconnect(
		EmaiSmtpSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Constants for Authentication modes
//				of the  EmaiSmtpAuthenticate function.
//
// @see		EmaiSmtpAuthenticate()
// @see		Emai::SmtpSession::Authenticate()
*/
enum EmaiEnumSmtpAuthSelector
{
	/*!< Automatically tries all available mechanisms*/
	EmaiSMTPAuthAutomatic,

	/*!< LOGIN authentification*/
	EmaiSMTPAuthLogin,

	/*!< PLAIN authentification*/
	EmaiSMTPAuthPlain,

	/*!< Cram-MD5 authentification*/
	EmaiSMTPAuthCramMD5,

	/*!< NTLM authentification*/
	EmaiSMTPAuthNTLM
};

/*!
// @brief	Authenticates the SMTP session.
//
// @param	inSession - session object, to be authenticated
// @param	inAccount - pointer to null-teminated string, which conatins
//						account name for authentication
// @param	inPassword - pointer to null-teminated string, which contains
//						password for authentication
// @param	inOptions - preffered authentication method, i.e.
//						EmaiSMTPAuthAutomatic, EmaiSMTPAuthLogin etc
//
// @see		EmaiEnumSmtpAuthSelector
// @see		Emai::SmtpSession::Authenticate
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpAuthenticate(
		EmaiSmtpSession inSession,
		EmaiConstUniCharPtr inAccount,
		EmaiConstUniCharPtr inPassword,
		EmaiOptions inOptions);

/*!
// @brief	Sends the message.
//
// @param	inSession - session object, to be used for sending
// @param	inMessage - message to be sent
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::SmtpSession::SendMessage
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpSendMessage(
		EmaiSmtpSession inSession,
		EmaiMessage inMessage,
		EmaiOptions inOptions);

/*!
// @brief	Sends the message.
//
// @param	inSession - session object, to be used for sending
// @param	inMessage - message to be sent
// @param	outFailedRecipients - failes recipients list
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::SmtpSession::SendMessage
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpSendMessageEx(
		EmaiSmtpSession inSession,
		EmaiMessage inMessage,
		EmaiStringList* outFailedRecipients,
		EmaiOptions inOptions);

/*!
// @brief	Marks the SMTP session to be cancelled ASAP.
//
// @param	inSession - session object, to be marked
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @see		Emai::SmtpSession::Cancel
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpCancel(
		EmaiSmtpSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Sends NOOP command to SMTP server, and reads server reply.
//			Used to avoid timout disconnection, during inactivity period.
//
// @param	inSession - session object
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpNoop(
		EmaiSmtpSession inSession,
		EmaiOptions inOptions);

/*!
// @brief	Sends a command to the SMTP server, reads server reply
//				and compares server reply code with the expected one
//				(passed via inExpectedReplyCode parameter).\n
//				Actual server answer can be accessed using 
//				EmaiSmtpGetLastServerResponse function.
// @param	inSession - session object, to be used for sending
// @param	inCommand - command to be sent to SMTP server
// @param	inExpectedReplyCode - expected server reply code, which means that
//			command was accepted by server succesfully
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Return value:
*/
EMAIAPI(EmaiError)
EmaiSmtpSendCommand(
		EmaiSmtpSession inSession,
		const char* inCommand,
		EmaiUint32 inExpectedReplyCode,
		EmaiOptions inOptions);

/*!
// @brief	Retrieves the last SMTP server response.
//
// @param	inSession - session object to be used
// @param	outResponse - on exit contains zero terminated last server response
// @param	ioResponseSize - specifies outResponse buffer size, on exit
//						filled with response length in bytes
// @param	outReplyCode - on exit contains reply code extracted from last
//						server response string, specfy NULL if you don't
//						need this infomation
// @param	inOptions - not used in current version, use EmaiNullOptions
//
// @return	Result code (See @ref EmaiEnumResultCodes)
*/
EMAIAPI(EmaiError)
EmaiSmtpGetLastServerResponse(
		EmaiSmtpSession inSession,
		char* outResponse,					/*can be NULL*/
		EmaiUint32* ioResponseSize,
		EmaiUint32* outReplyCode,			/*can be NULL*/
		EmaiOptions inOptions);

#ifdef __cplusplus
	}
#endif


#endif /*__EMAISMTP_H__*/
